from __future__ import print_function

import os
import sys

import paramiko
from six import BytesIO
from six.moves import input

from flag_bearer import utils


def plant(conf):
    """
    Remotely plant a flag.
    """
    resp = utils.get_user(conf)

    team = None
    red = resp['profile']['is_red']
    admin = resp['is_superuser']
    if red or admin:
        print("Which team do you want to get flags for?")
        try:
            team = int(input("> "))
        except ValueError:
            print("Retreiving flags for all teams")

    resp = utils.get_flags(conf, team)
    flags = {i: x for i, x in enumerate(resp)}

    print("Pick the flag to place")
    for flag_id, flag in flags.items():
        # Admins can see both blue and red flags, tell them which is which
        if admin:
            print("{}. {} ({})".format(flag_id, flag['name'], flag['type']))
        else:
            print("{}. {}".format(flag_id, flag['name']))

    flag = int(input("> "))

    if flag not in flags:
        print("[!!] Invalid selection")
        sys.exit(1)

    flag = flags[flag]
    data = flag['data'] + "\n"

    username = conf.cli_args.username
    host = conf.cli_args.host
    port = conf.cli_args.port
    location = conf.cli_args.location
    password = conf.cli_args.password

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(host, username=username, password=password, port=port)

    pkt = BytesIO()
    pkt.write(data.encode('utf-8'))
    pkt.seek(0)

    full_location = os.path.join(location, flag['filename'])
    sftp = ssh.open_sftp()
    sftp.putfo(pkt, full_location)
    print("Verifying flag plant")

    planted_contents = get_file_contents(ssh, full_location)

    # We need to get rid of trailing newlines, even though the planted
    # flag is supposed to have it, get_file_contents seems to remove it
    planted_contents = planted_contents.strip()
    data = data.strip()

    if isinstance(planted_contents, tuple):
        print("There was an error validating the capture:")
        print(planted_contents[1])
    elif planted_contents != data:
        print("Planted flag data does not match, double check the plant")
        diff = utils.get_diff(planted_contents, data)
        print(diff, end="")
    else:
        print("Flag Planted")


def get_file_contents(ssh, file):
    """
    Retrieve the contents of file from the remote server.
    """
    _, stdout, stderr = ssh.exec_command('cat {}'.format(file))
    err = stderr.read()
    if len(err) > 0:
        return False, err
    return stdout.read().decode('utf-8').strip()



from __future__ import print_function
from argparse import ArgumentParser
from flag_bearer import __version__, actions, config


parser = ArgumentParser()
parser.add_argument('--iscore-url', help="IScorE base url", default=None)
parser.add_argument('--api-version', default='v1')
parser.add_argument('--api-token')
subparsers = parser.add_subparsers()


plant = subparsers.add_parser('plant')
plant.set_defaults(func=actions.plant)
plant.add_argument('-f', '--flag', help="The name of the flag to plant")
plant.add_argument('-l', '--location', help="The location to plant the flag")
plant.add_argument('-n', '--team', help="The team number")
plant.add_argument('-s', '--save', action='store_true', default=False, 
                   help="Force the flag to be downloaded")


plant = subparsers.add_parser('download', help="Download all the flags for a team")
plant.set_defaults(func=actions.download)

try:
    import paramiko
    from flag_bearer import remote as remote_actions
    remote = subparsers.add_parser('remote', help='Remotely plant flags')
    remote_sub = remote.add_subparsers()

    remote_plant = remote_sub.add_parser('plant')
    remote_plant.set_defaults(func=remote_actions.plant)
    remote_plant.add_argument('-H', '--host', help='The host to connect to')
    remote_plant.add_argument('-p', '--port', help='The port of the remote host', default=22)
    remote_plant.add_argument('-u', '--username', help='The user to connect with')
    remote_plant.add_argument('-P', '--password', help='Password to connect with')
    remote_plant.add_argument('-l', '--location', help="The location to plant the flag", required=True)
except ImportError:
    pass


def main():
    print("Flag Bearer v{}".format(__version__))
    args = parser.parse_args()

    if not hasattr(args, 'func'):
        parser.print_help()
        return

    conf = config.Config.load()
    conf.merge(args)
    args.func(conf)




from six.moves import input
from six.moves.configparser import ConfigParser
from os.path import expanduser, dirname, join, exists
import getpass

ROOT = dirname(__file__)


class Config(ConfigParser):
    cli_args = None

    @classmethod
    def load(cls, noflagrc=False):
        # Load configuration files in order
        # - default.ini
        # - /etc/flagrc
        # - ~/.flagrc
        conf = cls()
        conf.read(join(ROOT, 'default.ini'))
        conf.read('/etc/flagrc')

        # Skip loading the flagrc, only used in tests
        if noflagrc:
            return conf

        flagrc = join(expanduser('~'), '.flagrc')
        if exists(flagrc):
            print("Loading {}".format(flagrc))
            conf.read(flagrc)

        return conf

    def merge(self, args):
        """
        Merge the configuration files with parsed arguments.
        """
        # Keep a copy of the args for an action
        self.cli_args = args
        if not self.has_section('iscore'):
            self.add_section('iscore')

        if args.iscore_url:
            self.set('iscore', 'base_url', args.iscore_url)

        if args.api_version:
            self.set('iscore', 'api_version', args.api_version)

        if hasattr(args, 'save') and args.save:
            self.set('iscore', 'force_save', 'yes')

        self.credentials = None
        if args.api_token:
            self.api_token = args.api_token
        elif self.has_option('iscore', 'api_token'):
            self.api_token = self.get('iscore', 'api_token')
        else:
            print("Enter your IScorE API Token (leave blank to use your credentials)")
            self.api_token = input("> ")

            if not self.api_token:
                print("Please login using your IScorE credentials")
                username = input("Username: ")
                password = getpass.getpass()
                self.credentials = (username, password)

    def request_extras(self):
        """
        Get extra requests configuration, specifically about authentication.

        Example:
        >>> extras = config.request_extras()
        >>> requests.get(url, **extras)
        """
        conf = {}
        if self.api_token:
            conf['headers'] = {
                'Authorization': 'Token {}'.format(self.api_token),
            }

        if self.credentials:
            conf['auth'] = self.credentials

        return conf
